/*  ___   ___  ___  _   _  ___   ___   ____ ___  ____  
 * / _ \ /___)/ _ \| | | |/ _ \ / _ \ / ___) _ \|    \ 
 *| |_| |___ | |_| | |_| | |_| | |_| ( (__| |_| | | | |
 * \___/(___/ \___/ \__  |\___/ \___(_)____)___/|_|_|_|
 *                  (____/ 
 * Osoyoo Wifi Arduino Robot   project
 * USe WI-FI UDP protocol to control 4-DOF robot arm
 * tutorial url: https://osoyoo.com/?p=39035
 */
 
#ifndef HAVE_HWSERIAL1
#include "SoftwareSerial.h"
SoftwareSerial Serial1(4, 5); // E_TX, E_RX
#endif
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>
#define finger_pin 15
#define wrist_pin 14
#define arm_pin 11
#define base_pin 10
// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

#define MIN_PULSE_WIDTH       650
#define MAX_PULSE_WIDTH       2350
#define DEFAULT_PULSE_WIDTH   1500
#define FREQUENCY             50
int arm_position=90;
int wrist_position=90;
int base_position=90;
#include "WiFiEspUdp.h"
#include "WiFiEsp.h"
WiFiEspUDP Udp;
unsigned int localPort = 8888;  // local port to listen on
 
#define led  8    //Right Motor direction pin 1 to MODEL-X IN2 yellow
 
 
char ssid[] = "***";   // replace *** with your router wifi SSID (name)
char pass[] = "***";   // replace *** with your router wifi password
char packetBuffer[5];      

// use a ring buffer to increase speed and reduce memory allocation
 
int status = WL_IDLE_STATUS;     // the Wifi radio's status
 
 
void setup()
{
  pwm.begin();
  pwm.setPWMFreq(FREQUENCY);  // Analog servos run at ~60 Hz updates

  Serial.begin(9600);   // initialize serial for debugging
  Serial1.begin(115200);    // initialize serial for ESP module
  Serial1.print("AT+CIOBAUD=9600\r\n");
  Serial1.write("AT+RST\r\n");
  Serial1.begin(9600);    // initialize serial for ESP module
  WiFi.init(&Serial1);    // initialize ESP module

  // check for the presence of the shield
  if (WiFi.status() == WL_NO_SHIELD) {
    Serial.println("WiFi shield not present");
    while (true); // don't continue
  }

 // Serial.print("Attempting to start AP ");
//  Serial.println(ssid);
  //AP mode
  //status = WiFi.beginAP(ssid, 10, "", 0);

//STA mode
   while ( status != WL_CONNECTED) {
    Serial.print("Attempting to connect to WPA SSID: ");
    Serial.println(ssid);
    // Connect to WPA/WPA2 network
    status = WiFi.begin(ssid, pass);
  }
 

  Serial.println("You're connected to the network");
  Serial.println();

  printWifiStatus();
  Udp.begin(localPort);
  Serial.print("Listening on port ");
  Serial.println(localPort);
 
 
}
 
 
void loop()
{   
 
  int packetSize = Udp.parsePacket();
  if (packetSize) {                               // if you get a client,
  //   Serial.print("Received packet of size ");
   // Serial.println(packetSize);
    int len = Udp.read(packetBuffer, 255);
    if (len > 0) {
      packetBuffer[len] = 0;
    }
      char c=packetBuffer[0];
      Serial.println(c);
            switch (c)    //serial control instructions
            {   
               case 'F':rotate(finger_pin,0);break;  //open finger
               case 'G':
                  arm_position+=5;
                  if(arm_position>180) arm_position=180;
		              rotate(arm_pin,arm_position);
                  break;  //open finger
               case 'H':
                  wrist_position+=5;
                  if(wrist_position>180) wrist_position=180;
                  rotate(wrist_pin, wrist_position);
                  break;  //open finger
               case 'I':rotate(finger_pin,180);break; //close finger
               case 'J':
                  arm_position-=5;
                  if(arm_position<0) arm_position=0;
                  rotate(arm_pin,arm_position);
                  break;
               case 'K':
                  wrist_position-=5;
                  if(wrist_position<0) wrist_position=0;
                  rotate(wrist_pin, wrist_position);
                  break;
               case 'L':
                  base_position-=5;
                  if(base_position<0) base_position=0;
		  rotate(base_pin,base_position);
                  break;
               case 'R':
                  base_position+=5;
                  if(base_position>180) base_position=180;
                  rotate(base_pin,base_position);
                  break;
               default:break;
              } //END OF ACTION SWITCH
  }
} //end of loop
void rotate(int servo_pin,int angle)
{
  int pulse_wide, analog_value;
  pulse_wide   = map(angle, 0, 180, MIN_PULSE_WIDTH, MAX_PULSE_WIDTH);
  analog_value = int(float(pulse_wide) / 1000000 * FREQUENCY * 4096);
  pwm.setPWM(servo_pin, 0, analog_value);
}

void printWifiStatus()
{
  // print your WiFi shield's IP address
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // print where to go in the browser
  Serial.println();
  Serial.print("To see this page in action, connect to ");
  Serial.print(ssid);
  Serial.print(" and open a browser to http://");
  Serial.println(ip);
  Serial.println();
}
